/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import org.w3c.dom.Element;
import org.w3c.dom.NodeList;

import java.util.*;

/**
 * Describes a single workflow
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.11 $
 */
public class WorkflowDescriptor
{
    private List initialSteps;
    private List steps;
    private List registers;
    private Map timerFunctions;

    public WorkflowDescriptor(Element root)
    {
        this.initialSteps = new ArrayList();
        this.steps = new ArrayList();
        this.registers = new ArrayList();
        this.timerFunctions = new HashMap();

        // handle registers
        Element r = (Element) root.getElementsByTagName("registers").item(0);
        if (r != null)
        {
            NodeList registers = r.getElementsByTagName("register");
            for (int i = 0; i < registers.getLength(); i++)
            {
                Element register = (Element) registers.item(i);
                this.registers.add(new RegisterDescriptor(register));
            }
        }

        // handle initial-steps
        Element intialStepsElement = (Element) root.getElementsByTagName("initial-steps").item(0);
        NodeList initialSteps = intialStepsElement.getElementsByTagName("initial-step");
        for (int i = 0; i < initialSteps.getLength(); i++)
        {
            Element initialStep = (Element) initialSteps.item(i);
            this.initialSteps.add(new InitialStepDescriptor(initialStep));
        }

        // handle timer-functions
        Element timerFunctionsElement = (Element) root.getElementsByTagName("trigger-functions").item(0);
        if(timerFunctionsElement!=null)
        {
            NodeList timerFunctions = timerFunctionsElement.getElementsByTagName("trigger-function");
            for (int i = 0; i < timerFunctions.getLength(); i++)
            {
                Element timerFunction = (Element) timerFunctions.item(i);
                Integer id = new Integer(timerFunction.getAttribute("id"));
                FunctionDescriptor function = new FunctionDescriptor((Element) timerFunction.getElementsByTagName("function").item(0));
                this.timerFunctions.put(id, function);
            }
        }

        // handle steps
        Element stepsElement = (Element) root.getElementsByTagName("steps").item(0);
        NodeList steps = stepsElement.getElementsByTagName("step");
        for (int i = 0; i < steps.getLength(); i++)
        {
            Element step = (Element) steps.item(i);
            this.steps.add(new StepDescriptor(step));
        }
    }

    /**
     * Get a List of initial steps for this workflow
     * @return A list of {@link InitialStepDescriptor} objects
     */
    public List getInitialSteps()
    {
        return initialSteps;
    }

    /**
     * Get a List of steps in this workflow
     * @return a List of {@link StepDescriptor} objects
     */
    public List getSteps()
    {
        return steps;
    }

    public StepDescriptor getStep(int id)
    {
        for (Iterator iterator = steps.iterator(); iterator.hasNext();)
        {
            StepDescriptor step = (StepDescriptor) iterator.next();
            if (step.getId() == id)
            {
                return step;
            }
        }

        return null;
    }

    public List getRegisters()
    {
        return registers;
    }

    public FunctionDescriptor getTriggerFunction(int id) {
        return (FunctionDescriptor) this.timerFunctions.get(new Integer(id));
    }
}
