/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.loader;

import com.opensymphony.util.Logger;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.xml.sax.SAXException;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;
import java.io.InputStream;
import java.net.URL;

/**
 * The WorkflowLoader is responsible for creating a WorkflowDesciptor
 * by loading the XML from various sources.
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.9 $
 */
public class WorkflowLoader
{
    private static final Logger log = new Logger(WorkflowLoader.class);

    /**
     * Load a workflow descriptor from a URL
     * @param url The URL of an XML document representing the workflow
     */
    public static WorkflowDescriptor load(URL url) throws SAXException, IOException
    {
        return load(url.openStream());
    }

    /**
     * Load a workflow descriptor from a resource
     * @param resource The path of an XML resource representing the workflow
     */
    public static WorkflowDescriptor load(String resource) throws SAXException, IOException
    {
        log.debug("Loading resource: " + resource);
        return load(Thread.currentThread().getContextClassLoader().getResourceAsStream(resource));
    }

    /**
     * Load a workflow descriptor from a file on disk
     * @param file The XML file representing the workflow
     */
    public static WorkflowDescriptor load(File file) throws SAXException, IOException
    {
        return load(new FileInputStream(file));
    }

    /**
     * Load a workflow descriptor from an InputStream
     */
    public static WorkflowDescriptor load(InputStream is) throws SAXException, IOException
    {
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        dbf.setNamespaceAware(true);
        DocumentBuilder db = null;

        try
        {
            db = dbf.newDocumentBuilder();
        }
        catch (ParserConfigurationException e)
        {
            log.fatal("Could not load workflow file", e);
        }

        Document doc = db.parse(is);

        Element root = (Element) doc.getElementsByTagName("workflow").item(0);

        return new WorkflowDescriptor(root);
    }
}
