/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ejb.step;

import com.opensymphony.module.user.EntityNotFoundException;
import com.opensymphony.module.user.User;
import com.opensymphony.module.user.UserManager;
import com.opensymphony.workflow.loader.StatusValues;
import com.opensymphony.ejb.EntityAdapter;

import javax.ejb.CreateException;
import java.sql.Timestamp;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.5 $
 */
public abstract class StepEJB extends EntityAdapter implements StatusValues {
    public abstract Long getId();

    public abstract void setId(Long id);

    public abstract Long getStepId();

    public abstract void setStepId(Long stepId);

    public abstract Integer getStatus();

    public abstract void setStatus(Integer statusId);

    public boolean isStatus(int status) {
        return (status == getStatus().intValue());
    }

    public boolean isFinished() {
        return isStatus(FINISHED);
    }

    public boolean isUnderway() {
        return isStatus(UNDERWAY);
    }

    public boolean isQueued() {
        return isStatus(QUEUED);
    }

    /**
     * markUnderway MUST be called first
     */
    public void markFinished() {
        if (isUnderway()) {
            setFinishDate(new Timestamp(System.currentTimeMillis()));
            setStatus(new Integer(FINISHED));
        }
    }

    public void markUnderway(User owner) {
        setStartDate(new Timestamp(System.currentTimeMillis()));
        setOwnerName(owner.getName());
        setStatus(new Integer(UNDERWAY));
    }

    public abstract Timestamp getStartDate();

    public abstract void setStartDate(Timestamp date);

    public abstract Timestamp getFinishDate();

    public abstract void setFinishDate(Timestamp date);

    public abstract Long getActionId();

    public abstract void setActionId(Long id);

    public abstract String getOwnerName();

    public abstract void setOwnerName(String owner);

    public User getOwner() {
      if(getOwnerName()==null) return null;
        try {
            UserManager um = UserManager.getInstance();
            return um.getUser(getOwnerName());
        } catch (EntityNotFoundException e) {
            return null;
        }
    }

    public abstract Long getPreviousId();

    public abstract void setPreviousId(Long id);

    public StepRemote getPrevious() {
        try {
            StepHome home = (StepHome) context.getEJBHome();
            Long pk = getPreviousId();
            if (pk != null && pk.longValue() != 0) {
                return home.findByPrimaryKey(pk);
            }
        } catch (Exception e) {
        }

        return null;
    }

    public abstract Long getEntryId();

    public abstract void setEntryId(Long id);

    /**
     * Create new step with no previous step.
     * This new step is marked as QUEUED.
     */
    public Long ejbCreate(Long entryId, long stepId) throws CreateException {
        try {
            Long id = new Long(nextLong());
            setId(id);
            setStepId(new Long(stepId));
            setStatus(new Integer(QUEUED));
            setEntryId(entryId);

            return id;
        } catch (Exception e) {
            throw new CreateException(e.getMessage());
        }
    }

    public void ejbPostCreate(Long entryId, long stepId) throws CreateException {
    }

    /**
     * Create a new step with no previous step.
     * This new step is marked as UNDERWAY.
     */
    public Long ejbCreate(Long entryId, long stepId, User owner) throws CreateException {
        try {
            Long id = new Long(nextLong());
            setId(id);
            setStepId(new Long(stepId));
            markUnderway(owner);
            setEntryId(entryId);

            return id;
        } catch (Exception e) {
            throw new CreateException(e.getMessage());
        }
    }

    public void ejbPostCreate(Long entryId, long stepId, User owner) throws CreateException {
    }

    /**
     * Create a new step with an REQUIRED previous step that MUST be marked finished.
     * This new step is marked as queued.
     */
    public Long ejbCreate(StepRemote prev, long stepId, long actionId) throws CreateException {
        try {
            if (prev == null || !prev.isFinished()) {
                throw new CreateException("Cannot add new step until previous step is finished!");
            }

            Long id = new Long(nextLong());
            setId(id);
            setStepId(new Long(stepId));
            setStatus(new Integer(QUEUED));


            prev.setActionId(new Long(actionId));
            setPreviousId(prev.getId());
            setEntryId(prev.getEntryId());

            return id;
        } catch (Exception e) {
            throw new CreateException(e.getMessage());
        }
    }

    public void ejbPostCreate(StepRemote prev, long stepId, long actionId) throws CreateException {
    }

    /**
     * Create a new step with a REQUIRED previous step that MUST be marked finished.
     * This new step is marked as UNDERWAY
     */
    public Long ejbCreate(StepRemote prev, long stepId, long actionId, User owner) throws CreateException {
        try {
            if (prev == null || !prev.isFinished()) {
                throw new CreateException("Cannot add new step until previous step is finished!");
            }

            Long id = new Long(nextLong());
            setId(id);
            setStepId(new Long(stepId));
            markUnderway(owner);

            prev.setActionId(new Long(actionId));
            setPreviousId(prev.getId());
            setEntryId(prev.getEntryId());

            return id;
        } catch (Exception e) {
            throw new CreateException(e.getMessage());
        }
    }

    public void ejbPostCreate(StepRemote prev, long stepId, long actionId, User owner) throws CreateException {
    }

    public Long ejbCreate(StepRemote prev, long actionId) throws CreateException {
        try {
            if (prev == null || !prev.isFinished()) {
                throw new CreateException("Cannot add new step until previous step is finished!");
            }

            Long id = new Long(nextLong());
            setId(id);
            setStepId(prev.getStepId());
            markUnderway(prev.getOwner());
            markFinished();

            prev.setActionId(new Long(actionId));
            setPreviousId(prev.getId());
            setEntryId(prev.getEntryId());

            return id;
        } catch (Exception e) {
            throw new CreateException(e.getMessage());
        }
    }

    public void ejbPostCreate(StepRemote prev, long actionId) throws CreateException {
    }
}
