/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ofbiz;

import com.opensymphony.module.user.User;
import com.opensymphony.workflow.persistence.Persistence;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.loader.StatusValues;
import com.opensymphony.workflow.WorkflowContext;
import com.opensymphony.util.Logger;
import org.ofbiz.core.entity.*;
import org.ofbiz.core.util.UtilMisc;

import java.util.HashMap;
import java.util.ArrayList;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.9 $
 */
public class OfbizPersistence implements Persistence, StatusValues {
    private static final Logger log = new Logger(OfbizPersistence.class);

    protected GenericDelegator gd = GenericDelegator.getGenericDelegator("default");
    private WorkflowContext context;
    private ArrayList list = new ArrayList();

    public OfbizPersistence(WorkflowContext context) {
        this.context = context;
    }

    public OfbizPersistence() {
        this.context = null;
    }

    public WorkflowEntry findWorkflowEntry(long pk) {
        try {
            return new OfbizWorkflowEntry(gd.findByPrimaryKey("workflowEntry", UtilMisc.toMap("id", new Long(pk))), context, this);
        } catch (GenericEntityException e) {
            log.error("Unable to find workflow entry #" + pk, e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public WorkflowEntry create(String workflowName) {
        Long pk = gd.getNextSeqId("step");

        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("workflowName", workflowName);

        GenericValue gv = gd.makeValue("workflowEntry", params);

        OfbizWorkflowEntry e = new OfbizWorkflowEntry(gv, context, this);

        markForStorage(gv);

        return e;
    }

    public Step create(long entryId, int stepId) {
        Long pk = gd.getNextSeqId("step");

        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("entryId", new Long(entryId));
        params.put("stepId", new Integer(stepId));
        params.put("status", new Integer(QUEUED));

        GenericValue gv = gd.makeValue("step", params);
        OfbizStep step = new OfbizStep(gv);

        markForStorage(gv);

        return step;
    }

    public Step create(long entryId, int stepId, User owner) {
        Long pk = gd.getNextSeqId("step");

        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("entryId", new Long(entryId));
        params.put("stepId", new Integer(stepId));

        GenericValue gv = gd.makeValue("step", params);
        OfbizStep step = new OfbizStep(gv);

        step.markUnderway(owner);

        markForStorage(gv);

        return step;
    }

    public Step create(Step prev, int stepId, int actionId) {
        OfbizStep ofbizPrev = (OfbizStep) prev;

        if (prev == null || !prev.isFinished()) {
            throw new RuntimeException("Cannot add new step until previous step is finished!");
        }

        Long pk = gd.getNextSeqId("step");
        prev.setActionId(actionId);

        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("status", new Integer(QUEUED));
        params.put("stepId", new Integer(stepId));
        params.put("actionId", new Long(actionId));

        GenericValue gv = gd.makeValue("step", params);
        OfbizStep step = new OfbizStep(gv);

        step.setPreviousId(prev.getId());
        step.setEntryId(prev.getEntryId());

        markForStorage(gv);
        markForStorage(ofbizPrev.gv);

        return step;
    }

    public Step create(Step prev, int stepId, int actionId, User owner) {
        OfbizStep ofbizPrev = (OfbizStep) prev;

        if (prev == null || !prev.isFinished()) {
            throw new RuntimeException("Cannot add new step until previous step is finished!");
        }

        Long pk = gd.getNextSeqId("step");
        prev.setActionId(actionId);

        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("status", new Integer(QUEUED));
        params.put("stepId", new Integer(stepId));

        GenericValue gv = gd.makeValue("step", params);

        OfbizStep step = new OfbizStep(gv);

        step.markUnderway(owner);

        step.setPreviousId(prev.getId());
        step.setEntryId(prev.getEntryId());

        markForStorage(gv);
        markForStorage(ofbizPrev.gv);

        return step;
    }

    public Step create(Step prev, int actionId) {
        OfbizStep ofbizPrev = (OfbizStep) prev;

        if (prev == null || !prev.isFinished()) {
            throw new RuntimeException("Cannot add new step until previous step is finished!");
        }

        Long pk = gd.getNextSeqId("step");
        prev.setActionId(actionId);


        HashMap params = new HashMap();
        params.put("id", pk);
        params.put("status", new Integer(QUEUED));
        params.put("stepId", new Integer(ofbizPrev.getStepId()));

        GenericValue gv = gd.makeValue("step", params);

        OfbizStep step = new OfbizStep(gv);

        step.markUnderway(prev.getOwner());
        step.markFinished();
        step.setPreviousId(prev.getId());
        step.setEntryId(prev.getEntryId());

        markForStorage(gv);
        markForStorage(ofbizPrev.gv);

        return step;
    }

    public Step findStep(long pk) {
        try {
            GenericValue gv = gd.findByPrimaryKey("step", UtilMisc.toMap("id", new Long(pk)));
            if (gv == null)
                return null;
            else
                return new OfbizStep(gv);
        } catch (GenericEntityException e) {
            log.error("Unable to find step #" + pk, e);
            throw new RuntimeException(e.getMessage());
        }
    }

    protected void markForStorage(GenericValue gv) {
        list.add(gv);
    }

    public void persistChanges() {
        try {
            gd.storeAll(list);
        } catch (GenericEntityException e) {
            log.error("Unable to persist changes", e);
            throw new RuntimeException(e.getMessage());
        }
    }
}
