/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.persistence.ofbiz;

import com.opensymphony.module.propertyset.PropertySet;
import com.opensymphony.module.propertyset.ofbiz.OFBizPropertySet;
import com.opensymphony.module.user.User;
import com.opensymphony.module.user.UserManager;
import com.opensymphony.module.user.EntityNotFoundException;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.WorkflowContext;
import com.opensymphony.util.Logger;
import org.ofbiz.core.entity.GenericValue;

/**
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.7 $
 */
public class OfbizWorkflowEntry implements WorkflowEntry {
    private static final Logger log = new Logger(OfbizWorkflowEntry.class);

    protected GenericValue gv;
    private OfbizPersistence p;
    private WorkflowContext context;

    public OfbizWorkflowEntry(GenericValue gv, WorkflowContext context, OfbizPersistence p) {
        this.gv = gv;
        this.p = p;
        this.context = context;
    }

    public void finishStep(int stepId, int actionId, User owner) {
        try {
            OfbizStep currentStep = (OfbizStep) p.findStep(gv.getLong("currentStep").longValue());

            if (currentStep.isQueued()) {
                User caller = UserManager.getInstance().getUser(context.getCaller());
                currentStep.markUnderway(caller);
            }

            if (currentStep.isUnderway()) {
                currentStep.markFinished();
            }

            OfbizStep newStep = (OfbizStep) p.create(currentStep, stepId, actionId, owner);

            gv.set("currentStep", new Long(newStep.getId()));

            newStep.markFinished();

            p.markForStorage(gv);
            p.markForStorage(currentStep.gv);
            p.markForStorage(newStep.gv);
        } catch (EntityNotFoundException e) {
            log.error("Can not finish step and assign to unknown user", e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public Step getCurrentStep() {
        Long pk = gv.getLong("currentStep");
        if (pk == null || pk.longValue() < 1) {
            return null;
        }

        return p.findStep(pk.longValue());
    }

    public long getId() {
        return gv.getLong("id").longValue();
    }

    public PropertySet getPropertySet() {
        return new OFBizPropertySet("workflowEntry", getId());
    }

    public String getWorkflowName() {
        return gv.getString("workflowName");
    }

    public void initializeStep(int stepId) {
        OfbizStep step = (OfbizStep) p.create(gv.getLong("id").longValue(), stepId);
        gv.set("currentStep", new Long(step.getId()));

        p.markForStorage(gv);
        p.markForStorage(step.gv);
    }

    public void initializeStep(int stepId, User owner) {
        OfbizStep step = (OfbizStep) p.create(gv.getLong("id").longValue(), stepId, owner);
        gv.set("currentStep", new Long(step.getId()));

        p.markForStorage(gv);
        p.markForStorage(step.gv);
    }

    public void newStep(int stepId, int actionId) {
        try {
            OfbizStep currentStep = (OfbizStep) getCurrentStep();

            if (currentStep.isQueued())
            {
                // since it was never even started, make it underway as the current caller
                User caller = UserManager.getInstance().getUser(context.getCaller());
                currentStep.markUnderway(caller);
            }

            if (currentStep.isUnderway())
            {
                currentStep.markFinished();
            }

            OfbizStep step = (OfbizStep) p.create(currentStep, stepId, actionId);
            gv.set("currentStep", new Long(step.getId()));

            p.markForStorage(gv);
            p.markForStorage(currentStep.gv);
            p.markForStorage(step.gv);
        } catch (EntityNotFoundException e) {
            log.error("Can not finish step with an unknown caller", e);
            throw new RuntimeException(e.getMessage());
        }
    }

    public void newStep(int stepId, int actionId, User owner) {
        try {
            OfbizStep currentStep = (OfbizStep) getCurrentStep();

            if (currentStep.isQueued())
            {
                // since it was never even start, make it underway as the current caller
                User caller = UserManager.getInstance().getUser(context.getCaller());
                currentStep.markUnderway(caller);
            }

            if (currentStep.isUnderway())
            {
                currentStep.markFinished();
            }

            OfbizStep step = (OfbizStep) p.create(currentStep, stepId, actionId, owner);
            gv.set("currentStep", new Long(step.getId()));

            p.markForStorage(gv);
            p.markForStorage(currentStep.gv);
            p.markForStorage(step.gv);
        } catch (EntityNotFoundException e) {
            log.error("Cannot create new step when the caller is not found", e);
            throw new RuntimeException(e.getMessage());
        }
    }
}
