/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.util;

import com.opensymphony.util.Logger;
import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.persistence.WorkflowEntry;

import javax.naming.InitialContext;
import javax.rmi.PortableRemoteObject;
import java.lang.reflect.Method;
import java.util.Map;
import java.util.Hashtable;

/**
 * Generic EJB Invoker function.
 * This function is used to invoke an EJB listener in a step. The EJB
 * must implement WorkflowListener if it's a remote session bean, or
 * WorkflowLocalListener if it's a local session bean.<br>
 * It accepts a number of arguments, these are:
 *
 * <ul>
 *  <li>ejb-home - The fully qualified class name of the EJB remote home interface</li>
 *  <li>ejb-local-home - The fully qualified class name of the local home interface</li>
 *  <li>ejb-jndi-location - The JNDI location of the ejb to invoke</li>
 * </ul>
 * <p>
 * Note that only one of ejb-home or ejb-local-home can be specified.
 *
 * Also, please note that the entire set of properties will be passed through to the
 * constructor for InitialContext, meaning that if you need to use an
 * InintialContextFactory other than the default one, you are free to include arguments
 * that will do so.
 *
 * @author Hani Suleiman
 * @version $Revision: 1.8 $
 * Date: Apr 6, 2002
 * Time: 11:48:14 PM
 */
public class EJBInvoker implements FunctionProvider
{
    private static final Logger log = new Logger(EJBInvoker.class);

    public void execute(Map inputs, Map properties, Map variables)
    {
        Class homeClass = null;
        Object home = null;
        WorkflowEntry entry = (WorkflowEntry) variables.get("entry");
        Hashtable env = new Hashtable(properties);
        try
        {
            InitialContext ic = new InitialContext(env);
            log.debug("executing with properties=" + properties);
            if (properties.containsKey("ejb-home"))
            {
                homeClass = Class.forName((String) properties.get("ejb-home"));
            }
            else if (properties.containsKey("ejb-local-home"))
            {
                homeClass = Class.forName((String) properties.get("ejb-local-home"));
            }
            home = PortableRemoteObject.narrow(ic.lookup((String) properties.get("ejb-jndi-location")), homeClass);
            Method method = homeClass.getMethod("create", new Class[0]);
            if (java.rmi.Remote.class.isAssignableFrom(homeClass))
            {
                WorkflowListener listener = (WorkflowListener) method.invoke(home, new Object[0]);
                listener.stateChanged(entry);
            }
            else
            {
                WorkflowLocalListener listener = (WorkflowLocalListener) method.invoke(home, new Object[0]);
                listener.stateChanged(entry);
            }
        }
        catch (Exception e)
        {
            log.error("Error invoking EJB homeClass=" + homeClass, e);
        }
    }
}
