/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.util;

import com.opensymphony.util.Logger;
import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.persistence.WorkflowEntry;

import javax.jms.*;
import javax.naming.InitialContext;
import java.util.Map;
import java.util.Hashtable;
import java.util.Iterator;

/**
 * Sends out a JMS TextMessage to a specified Queue or Topic. The following arguments
 * are expected:
 *
 * <ul>
 *  <li>queue-factory-location - the location to be passed to InitialContext.lookup</li>
 *  <li>queue-location - the location to be passed to InitialContext.lookup</li>
 *  <li>topic-factory-location - the location to be passed to InitialContext.lookup</li>
 *  <li>topic-location - the location to be passed to InitialContext.lookup</li>
 *  <li>text - the text message to be included in this JMS message</li>
 * </ul>
 *
 * Also, please note that the entire set of properties will be passed through to the
 * constructor for InitialContext, meaning that if you need to use an
 * InintialContextFactory other than the default one, you are free to include arguments
 * that will do so.
 *
 * Also note that all arguments are also passed to the TextMessage using
 * setObjectProperty(), except for "text" which is set using setText().
 *
 * @author Hani Suleiman
 * @version $Revision: 1.9 $
 */
public class JMSMessage implements FunctionProvider
{
    private static final Logger log = new Logger(JMSMessage.class);

    public void execute(Map inputs, Map properties, Map variables)
    {
        WorkflowEntry entry = (WorkflowEntry) variables.get("entry");
        try
        {
            Hashtable env = new Hashtable(properties);
            InitialContext initialContext = new InitialContext(env);

            if (properties.containsKey("queue-factory-location"))
            {
                QueueConnectionFactory queueFactory = (QueueConnectionFactory) initialContext.lookup((String) properties.get("queue-factory-location"));
                QueueConnection conn = queueFactory.createQueueConnection();
                conn.start();
                QueueSession queueSession = conn.createQueueSession(false, Session.AUTO_ACKNOWLEDGE);
                Queue queue = (Queue) initialContext.lookup((String) properties.get("queue-location"));
                QueueSender sender = queueSession.createSender(queue);
                TextMessage message = queueSession.createTextMessage();
                populateMessage(message, entry, properties);
                sender.send(message);
            }
            else if (properties.containsKey("topic-factory-location"))
            {
                TopicConnectionFactory topicFactory = (TopicConnectionFactory) initialContext.lookup((String) properties.get("topic-factory-location"));
                TopicConnection conn = topicFactory.createTopicConnection();
                conn.start();
                TopicSession topicSession = conn.createTopicSession(false, Session.AUTO_ACKNOWLEDGE);
                Topic topic = (Topic) initialContext.lookup((String) properties.get("topic-location"));
                TopicPublisher publisher = topicSession.createPublisher(topic);
                TextMessage message = topicSession.createTextMessage();
                populateMessage(message, entry, properties);
                publisher.publish(message);
            }
        }
        catch (Exception ex)
        {
            log.error("Error sending JMS message", ex);
        }
    }

    private void populateMessage(TextMessage message, WorkflowEntry entry, Map properties) throws JMSException
    {
        message.setText((String) properties.get("text"));
        message.setObjectProperty("workflowEntry", entry);

        for (Iterator iterator = properties.entrySet().iterator(); iterator.hasNext();) {
            Map.Entry mapEntry = (Map.Entry) iterator.next();

            // don't include "text", it was already done
            if (!"text".equals(mapEntry.getKey())) {
                message.setObjectProperty((String) mapEntry.getKey(), mapEntry.getValue());
            }
        }
    }
}
