/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.util;

import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.WorkflowContext;
import com.opensymphony.workflow.InternalWorkflowException;
import com.opensymphony.workflow.persistence.Step;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.util.TextUtils;

import java.util.Map;

/**
 * Sets the persistent variable "mostRecentOwner" to the owner of the most
 * recent step that was of type stepId. If quiet mode is enabled, then the
 * variable is set to "". Otherwise, the transaction is rolled back and an
 * InternalWorkflowException is thrown. This function accepts the following
 * arguments:
 *
 * <ul>
 *  <li>stepId - the stepId of the most recent step to look for (required)</li>
 *  <li>quietMode - if "true", then no error is thrown if a user can't e found (optional, defaults to <b>false</b>)</li>
 * </ul>
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.10 $
 */
public class MostRecentOwner implements FunctionProvider {
    public void execute(Map inputs, Map properties, Map variables) {
        // expects a stepId name/value pair
        String quietModeS = (String) properties.get("quietMode");
        boolean quietMode = false;
        quietMode = TextUtils.parseBoolean(quietModeS);
        String stepIdString = (String) properties.get("stepId");
        WorkflowEntry entry = (WorkflowEntry) variables.get("entry");
        WorkflowContext context = (WorkflowContext) variables.get("context");

        if (stepIdString == null) {
            throw new InternalWorkflowException("This function expects a stepId name/value pair!");
        }

        int stepId = Integer.parseInt(stepIdString);

        try {
            Step prev = entry.getCurrentStep().getPrevious();
            variables.put("mostRecentOwner", "");

            while (prev != null) {
                if (prev.getStepId() == stepId) {
                    variables.put("mostRecentOwner", prev.getOwner().getName());
                    return;
                }

                prev = prev.getPrevious();
            }
        } catch (Exception e) {
            if (!quietMode) {
                context.setRollbackOnly();
                throw new InternalWorkflowException("Unable to determine owner of most recent step with ID " + stepId, e);
            }
        }
    }
}
