/*
 * Created by IntelliJ IDEA.
 * User: plightbo
 * Date: May 22, 2002
 * Time: 4:05:53 PM
 */
package com.opensymphony.workflow.util;

import com.opensymphony.workflow.FunctionProvider;
import com.opensymphony.workflow.persistence.WorkflowEntry;
import com.opensymphony.workflow.timer.WorkflowJob;
import com.opensymphony.util.Logger;

import java.util.Map;
import java.util.Date;

import org.quartz.*;
import org.quartz.impl.StdSchedulerFactory;

/**
 * Schedules a job in the Quartz job scheduler to be executed one or more times in the future.
 * The following arguments are required:
 *
 * <ul>
 *  <li> triggerId - the id of the trigger function defined in the XML workflow
 *  <li> jobName - the name to be given to the job
 *  <li> triggerName - the name to be given to the trigger
 *  <li> groupName - the group given to both the job and the trigger
 *  <li> username - the system account's username that will execute the function in the future
 *  <li> password - the system account's password
 * </ul>
 *
 * If you are using a cron trigger, the following is required:
 * <ul>
 *  <li> cronExpression - the Cron expression
 * </ul>
 *
 * If you are using a simple trigger, the follow are all optional:
 * <ul>
 *  <li> startOffset - the offset, in milliseconds, from the current time. (default is 0)
 *  <li> endOffset - the offset, in milliseconds, from the current time. (default is infinity)
 *  <li> repeat - the repeat count (default is 0)
 *  <li> repeatDelay - the time delay, in milliseconds, between repeats (default is 0)
 * </ul>
 */
public class ScheduleJob implements FunctionProvider {
    private static final Logger log = new Logger(ScheduleJob.class);

    public void execute(Map inputs, Map properties, Map variables) {
        try {
            WorkflowEntry entry = (WorkflowEntry) variables.get("entry");

            log.info("Starting to schdule job for WF #" + entry.getId());

            int triggerId = Integer.parseInt((String) properties.get("triggerId"));
            String jobName = (String) properties.get("jobName");
            String triggerName = (String) properties.get("triggerName");
            String groupName = (String) properties.get("groupName");

            String username = (String) properties.get("username");
            String password = (String) properties.get("password");

            String cronExpression = (String) properties.get("cronExpression");

            jobName = jobName + ":" + entry.getId();
            triggerName = triggerName + ":"+ entry.getId();
            groupName = groupName + ":" + entry.getId();

            Scheduler s = new StdSchedulerFactory().getScheduler();

            JobDetail jobDetail = new JobDetail(jobName, groupName, WorkflowJob.class);
            Trigger trigger = null;

            if (cronExpression == null) {
                long now = System.currentTimeMillis();

                // get start date - default is now
                Date startDate = null;
                try {
                    String start = (String) properties.get("startOffset");
                    if (s != null) {
                        startDate = new Date(now + Long.parseLong(start));
                    }
                } catch (NumberFormatException e) {
                }
                if (startDate == null) {
                    startDate = new Date(now);
                }

                // get end date - default is null
                Date endDate = null;
                try {
                    String end = (String) properties.get("endOffset");
                    if (s != null) {
                        startDate = new Date(now + Long.parseLong(end));
                    }
                } catch (NumberFormatException e) {
                }

                // get the repeat amount - default is 0
                int repeat = 0;
                try {
                    String r = (String) properties.get("repeat");
                    if (r != null) {
                        if (r.equalsIgnoreCase("REPEAT_INDEFINITELY")) {
                            repeat = SimpleTrigger.REPEAT_INDEFINITELY;
                        } else {
                            repeat = Integer.parseInt(r);
                        }
                    }
                } catch (NumberFormatException e) {
                }

                // get repeat delay - default is 0
                long delay = 0;
                try {
                    String rd = (String) properties.get("repeatDelay");
                    if (rd != null) {
                        delay = Long.parseLong(rd);
                    }
                } catch (NumberFormatException e) {
                }

                trigger = new SimpleTrigger(triggerName, groupName, jobName, groupName, startDate, endDate, repeat, delay);
            } else {
                trigger = new CronTrigger(triggerName, groupName, jobName, groupName, cronExpression);
            }


            JobDataMap dataMap = new JobDataMap();
            dataMap.put("triggerId", triggerId);
            dataMap.put("entryId", entry.getId());
            dataMap.put("username", username);
            dataMap.put("password", password);
            jobDetail.setJobDataMap(dataMap);

            s.scheduleJob(jobDetail, trigger);

            log.info("Job scheduled");
        } catch (Exception e) {
            log.error("Error scheduling job", e);
        }
    }
}
