/*
 * This software is published under the terms of the OpenSymphony Software
 * License version 1.1, of which a copy has been included with this
 * distribution in the LICENSE.txt file.
 */
package com.opensymphony.workflow.util;

import com.opensymphony.util.Logger;
import com.opensymphony.workflow.FunctionProvider;

import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.InternetAddress;
import javax.mail.internet.MimeMessage;
import java.util.*;

/**
 * Sends an email to a group of users. The following arguments are expected:
 *
 * <ul>
 *  <li>to - comma seperated list of email addresses</li>
 *  <li>from - single email address</li>
 *  <li>subject - the message subject</li>
 *  <li>cc - comma seperated list of email addresses (optional)</li>
 *  <li>message - the message body</li>
 *  <li>smtpHost - the SMTP host that will relay the message</li>
 * </ul>
 *
 * @author <a href="plightbo@users.sourceforge.net">Patrick Lightbody</a>
 * @version $Revision: 1.12 $
 */
public class SendEmail implements FunctionProvider
{
    private static final Logger log = new Logger(SendEmail.class);

    public void execute(Map inputs, Map properties, Map variables)
    {
        String to = (String) properties.get("to");
        String from = (String) properties.get("from");
        String subject = (String) properties.get("subject");
        String cc = (String) properties.get("cc");
        String m = (String) properties.get("message");
        String smtpHost = (String) properties.get("smtpHost");

        log.debug("Host: " + smtpHost);
        log.debug("To: " + to);
        log.debug("Cc: " + cc);
        log.debug("From: " + from);
        log.debug("Subject: " + subject);
        log.debug("Message: " + m);

        try
        {
            Properties props = new Properties();
            props.put("mail.smtp.host", smtpHost);
            Session sendMailSession = Session.getInstance(props, null);
            Transport transport = sendMailSession.getTransport("smtp");
            Message message = new MimeMessage(sendMailSession);

            message.setFrom(new InternetAddress(from));
            Set toSet = new HashSet();
            StringTokenizer st = new StringTokenizer(to, ", ");

            while (st.hasMoreTokens())
            {
                String user = st.nextToken();
                toSet.add(new InternetAddress(user));
            }

            message.setRecipients(Message.RecipientType.TO, (InternetAddress[]) toSet.toArray(new InternetAddress[toSet.size()]));
            Set ccSet = new HashSet();
            st = new StringTokenizer(cc, ", ");

            while (st.hasMoreTokens())
            {
                String user = st.nextToken();
                ccSet.add(new InternetAddress(user));
            }

            message.setRecipients(Message.RecipientType.CC, (InternetAddress[]) ccSet.toArray(new InternetAddress[ccSet.size()]));
            message.setSubject(subject);
            message.setSentDate(new Date());
            message.setText(m);
            message.saveChanges();

            transport.connect();
            transport.sendMessage(message, message.getAllRecipients());
            transport.close();
        }
        catch (MessagingException e)
        {
            log.error("Error sending email:", e);
        }
    }
}
